'''
    aml.path.path
    ~~~~~~~~~~~~~

    :copyright: 2009 by Bjoern Schulz <bjoern.schulz.92@gmail.com>
    :license: MIT, see LICENSE for more details
'''

class Path(object):
    
    def __init__(self, name, attributes=None, child=None):
        self.name = name
        self.attributes = attributes or {}
        self.child = child
    
    def __repr__(self):
        return '{0}({1}, {2}, {3})' % (
            self.__class__.__name__,
            repr(self.name),
            repr(self.attributes),
            repr(self.child)
        )
    
    def __str__(self):
        return '{0}[{1}]/{2}' % (
            self.name,
            ';'.join('{0}={1}' % (i[0], repr(i[0])) for i in self.attributes.items()),
            self.child or ''
        )
    
    def match(self, node):
        return (
            (node.name == self.name or self.name == '*') and
            all(node[k] == self.attributes[k] for k in self.attributes)
        )
    
    def findall(self, node):
        if self.match(node):
            if self.child is None:
                yield node
            else:
                for child in node:
                    for n in self.child.findall(child):
                        yield n

class Root(Path):
    
    def __str__(self):
        return '/{0}[{1}]/{2}' % (
            self.name,
            ';'.join('{0}={1}' % (i[0], repr(i[0])) for i in self.attributes.items()),
            self.child or ''
        )
    
    def root(self, node):
        while node.parent is not None:
            node = node.parent
        return node
    
    def findall(self, node):
        node = self.root(node)
        if self.match(node):
            if self.child is None:
                yield node
            else:
                for child in node:
                    for n in self.child.findall(child):
                        yield n

class Any(Path):
    
    def __str__(self):
        return '//{0}[{1}]/{2}' % (
            self.name,
            ';'.join('{0}={1}' % (i[0], repr(i[0])) for i in self.attributes.items()),
            self.child or ''
        )
    
    def flatten(self, node):
        yield node
        for n in node:
            for r in self.flatten(n):
                yield r
    
    def findall(self, node):
        for n in self.flatten(node):
            if self.match(n):
                if self.child is None:
                    yield n
                else:
                    for r in self.child.findall(n):
                        yield r

class Self(object):
    
    def __init__(self, child=None):
        self.child = child
    
    def __repr__(self):
        return 'Self({0})' % (
            repr(self.child)
        )
    
    def __str__(self):
        return './{0}' % (
            self.child or ''
        )
    
    def findall(self, node):
        if self.child is None:
            yield node
        else:
            for child in node:
                for n in self.child.findall(child):
                    yield n

class Parent(object):
    
    def __init__(self, child=None):
        self.child = child
    
    def __repr__(self):
        return 'Parent({0})' % (
            repr(self.child)
        )
    
    def __str__(self):
        return '../{0}' % (
            self.child or ''
        )
    
    def findall(self, node):
        if self.child is None:
            if node.parent is None:
                yield node
            else:
                yield node.parent
        else:
            if node.parent is not None:
                node = node.parent
            for child in node:
                for n in self.child.findall(child):
                    yield n